@rem = '--*-Perl-*--
@echo off
REM ######################################################################
REM Self-start wrapper code for Windows 

set PATH_SEARCH=
if NOT EXIST %0 set PATH_SEARCH=-S

if "%OS%" == "Windows_NT" goto Windows
ratlperl -x %PATH_SEARCH% %0 %*
if errorlevel 1 exit %errorlevel%
goto endofperl

:Windows
ratlperl -x %PATH_SEARCH% %0 %*
if NOT "%COMSPEC%" == "%SystemRoot%\system32\cmd.exe" goto endofperl
if %errorlevel% == 9009 echo You do not have ratlperl in your PATH.
if errorlevel 1 goto script_failed_so_exit_with_non_zero_val 2>nul
goto endofperl

REM ######################################################################
@rem ';

#!/opt/rational/common/bin/ratlperl
#
# Licensed Materials - Property of IBM
# Rational ClearCase
# (C) Copyright IBM Corp. 2006, 2007. All Rights Reserved.
# US Government Users Restricted Rights
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#:cqcc_cache_mgr - 
# This script launches the central cache manager script 
# that checks the contents of the central cache directory used by the 
# Base CC/CQ integration. It is intended to be run as a task with the 
# ClearCase Scheduler either as a regularly scheduled job or as a single
# run. 
# 
# It does the following: 
# o redirects output to the appropriate log file location
# o sets up run time parameters to control how the cache manager will run, 
# o starts the cache manager using cqcc_launch so that it will run using 
#   the sites config.pl file and trigger source code. 
# o reports if the cache manager succeeded overall or not

use strict;
require Config;
use File::Basename;

######################################################################
#::Global values

my $windows = ($Config::Config{osname} eq "MSWin32") ? 1 : 0;
my $ps = ($windows ? "\\" : "/");  #Path Separator
my $logFile = "";
my $logSizeMax = 1000;   # Default max log size before renaming to .old (1 Meg)

# ArgDescriptions provide the name of the argument and a string
# describing if the option is a "LocalOpt" (local option for this script)
# or an "Opt" (passthru option for TriggerCacheMgr) and then a help
# string consisting of the argument value type and a comment.
# 
# Note: Lining up the last quote on the arg name with the 
# first quote on the comment line then aligning the #'s 
# leads to an aligned help output. 

my %argDesc = (
    "central"  => "LocalOpt:" .
            "<dir>               # Location of integration source",

    "config"   => "LocalOpt:" .
           "<file>               # Name of integration config file",

    "cqbatch_post_mins"  => "Opt:" .
                      "<minutes> # Time before posting unposted CQ batches\n".
    "                             #  (default 60 mins, min 3, max 6000)",

    "cqlogin"  => "Opt:" .
            "<name:password>     # User name and encrypted password\n".
    "                             #  taken from existing .cqparams file",

    "dir_stale_hrs"  => "Opt:" .
                  "<hours>       # Time before deleting unused directories\n" .
    "                             #  (default 48 hrs, min 8, max 1000)",

    "env"      => "LocalOpt:" .
        "<name:value>            # Set environment variable name:value pair\n" . 
        "                             #  May be used multiple times",

    "help"     => "LocalFlag:" . 
         "                       # Print out list of command line options",

    "maxlogsize"  => "LocalOpt:" .
               "<filesizeK>      # Max size of log file before renaming to .old",

    "noexec"   => "Flag:" .
           "                     # Runs cache manager in no-execute mode",

    "nolog"    => "LocalFlag:" .
          "                      # Runs cache manager with output going to stdout",

    "path"     => "Opt:" .
         "<dir>                  # Run against specific cache subdirectory",

    "user"     => "Opt:" .
         "<dir>                  # Run against specific user's subdirectory",

    "stale_hrs"=> "Opt:" .
              "<hours>           # Time before stale series cache and \n" .
    "                             #  cqsession files should be deleted \n" .
    "                             #  individually regardless of directory age\n" .
    "                             #  (default 8, min 0.1, max 100) ",

    "verbose"  => "Opt:" .
            "<level>             # Verbose output level 0 through 3\n" .
    "                             #  0(errors only), 1(dirs), 2(files), 3(debug)",

    "vob"      => "LocalOpt:" .
        "<vobname>               # Name of vob where triggers are installed",
     );

my %argValue = ();

######################################################################
#::main
# Redirect stdout and stdin for duration of the operation 
##################################################

sub main {
    my $result = 0;

    $ENV{CQCC_GUI_ENABLE}="OFF"; # Avoid using GUI dialogs during cache mgmt
    $ENV{CQCC_NO_STDIN} = 1;   # Cause fatal error if need to prompt user

    ##################################################
    # Determine logFile location 

    my $logFile = "";
    my $i;
    # Pre-process args looking for nolog and maxlogsize 
    for ($i = 0; $i <= $#ARGV; $i++) {
        if ($ARGV[$i] eq "-maxlogsize") {
            $i++;
            $logSizeMax = $ARGV[$i];
        } elsif ($ARGV[$i] eq "-nolog") {
            $argValue{"nolog"} = 1;
        }
    }

    if ($argValue{"nolog"} ne "1") {
        my $logDir = "";
        if ($windows) {
            if (exists $ENV{CLEARCASEHOME}) {
                $logDir = "$ENV{CLEARCASEHOME}\\var\\log";
            }
        } else {
            $logDir = "/var/adm/rational/clearcase/log";
        }
        if ($logDir ne "") {
            $logFile = $logDir . $ps . "cqcc_cache_mgr_log";
        }
        # Check existing logFile for exceeding maximum size before output
        if ($logFile ne "") {
            my @fileInfo = stat($logFile);
            my $fileSize = $fileInfo[7];
            if ($fileSize > ($logSizeMax * 1024)) {
                rename_file($logFile,$logFile . ".old");
            }
        }
    } 

    if ($logFile ne "") {
        open(SAVED_STDOUT, ">&STDOUT") || 
            die("can't dup stdout\n");
        close(STDOUT);
        open(STDOUT, ">>$logFile") || 
            die("can't redirect STDOUT to $logFile\n");

        open(SAVED_STDERR, ">&STDERR") || 
            die("can't dup stdout\n");
        close(STDERR);
        open(STDERR, ">&STDOUT") || 
            die("can't redirect STDERR to STDOUT\n");
    }

    my $time = localtime(time);
    print "---------------------------------------------------\n" . 
          "Base CC/CQ Integration Central Cache Manager: $time\n\n";

    $result = operation();

    # Restore original stdout 
    if ($logFile ne "") {
        close(STDOUT);
        open(STDOUT, ">&SAVED_STDOUT");
        close(SAVED_STDOUT);

        close(STDERR);
        open(STDERR, ">&SAVED_STDERR");
        close(SAVED_STDERR);
    }
    exit $result;
}

######################################################################
#::operation routine

sub operation {
    my @manageOptions = ();

    ##################################################
    # Process command line arguments
    
    while (@ARGV) {
        my $arg = shift @ARGV;
        my $valid = 0;
        if (substr($arg,0,1) eq "-") {
            my $key = substr($arg,1);
            my $argDef = $argDesc{$key};
            my ($prop,$doc) = split(":",$argDef,2);
            $valid = 1;
            if ($prop =~ /Opt/) {
                my $value = shift @ARGV;
                if (! defined $value || substr($value,0,1) eq "-") {
                    print "Error: Command line argument expects value\n" .
                          "$arg $doc\n";
                    return 1;
                }
                if ($key eq "env") {
                    my ($ename,$evalue) = split(":",$value,2);
                    $evalue =~ s/\"//g;
                    $ENV{$ename} = $evalue;
                }
                $argValue{$key} = $value;
                if ($argDef !~ /Local/) {
                    push @manageOptions, "CM_" . uc($key) . "=$value";
                }
            } elsif ($argDef =~ /Flag/) {
                $argValue{$key} = 1;
                if ($argDef !~ /Local/) {
                    push @manageOptions, "CM_" . uc($key) . "=1";
                }
            } else {
                $valid = 0;
            }
        }
        if ($valid ne "1") {
            print "Error: Unexpected command line argument ($arg)\n" .
                  "command line options include the following\n";
        }
        if ($valid ne "1" || $argValue{"help"} eq "1") {
            print "$0
launches the central cache manager script for the Base CC/CQ integration.
You must provide either the -vob option or the -config/-central options 
to identify where the integration source and config file reside so this 
tool can find the central cache and other parameters. 

Use -noexec to see what the tool would do without actually changing files

It takes the following arguments:\n\n";

            for (sort keys %argDesc) {
                my ($prop,$doc) = split(":",$argDesc{$_},2);
                print "-$_ $doc\n";
            }
            return 1;
        }
    }
 
    if (defined $argValue{"cqlogin"}) {
        my $cqLogin = $argValue{"cqlogin"};
        my @parts = split(":",$cqLogin,2);
        if ($#parts < 1) {
            print "Error: CQLogin argument expects name and password " .
                  " separated by :\n";
            my ($prop,$doc) = split(";",$argDesc{"cqlogin"},2);
            print "$doc\n";
            return 1;
        }
    }

    ##################################################
    # If sampleVob is set determine centralDir and configFile
    # from checkin trigger

    my $sampleVob = $argValue{"vob"};
    my $binPath = $windows ? "" : "/opt/rational/clearcase/bin/";

    if ($sampleVob ne "") {
        my $ct = $binPath . "cleartool";
        my @results = 
            `$ct describe trtype:cq_ci_trigger\@$sampleVob`;

        for (@results) {
            my $line = $_;
            if (($windows && $line =~ /execwin/) || 
                (! $windows && $line =~ /execunix/)) {
                if ($line =~ /cqcc_launch\S*\s(\S*)\s/) {
                    if (substr($1,0,1) ne "-") {
                        $argValue{"config"} = $1;
                    }
                }
                if ($line =~ /\-central\s(\S*)\s/) {
                    $argValue{"central"} = $1;
                }
            }
        }
    }

    ##################################################
    # Check for sufficient information to run

    my $configFile = $argValue{"config"};
    my $centralDir = $argValue{"central"};

    if ($centralDir eq "" && $sampleVob eq "") {
        print "Error: Insufficient information to run. " .
              "Must set one of the following options:\n" . 
              "-central or -vob\n";
        return 1;    
    }

    ##################################################
    # Determine launch command 

    my $launch = ($centralDir ? $centralDir . $ps : $binPath) .
                 "cqcc_launch";

    if ($launch =~ /\s/) {
        $launch = "\"$launch\"";
    }
    if ($windows eq "1") {
        $launch =~ s/\//\\/g;
    }
    my $centralDirOption = "";
    if ($centralDir ne "") {
        if ($windows eq "1") {
            $centralDir =~ s/\//\\/g;
        }
        if ($centralDir =~ /\s/) {
            $centralDir = "\"$centralDir\"";
        }
        $centralDirOption = "-central $centralDir";
    }
    my $cmd = "$launch $configFile $centralDirOption " . 
        "-p TriggerInterface,CQCC::TriggerCacheMgr " . 
        "-op manage " . join(",",@manageOptions);

    print "$cmd\n";

    my $result = system($cmd);

    print "Result: $result\n";
    return $result;
} 

######################################################################
#::rename_file

sub rename_file {
    my ($filename,$newName) = @_;
    my $cmd;
    if (-e $newName) {
        unlink $newName;
    }
    if ($windows) {
        my $base = basename($newName);
        $cmd = "rename \"$filename\" \"$base\"";
    } else {
        my $dirOld = dirname($filename);
        my $baseOld = basename($filename);
        my $dirNew = dirname($newName);
        my $baseNew = basename($newName);
        $cmd = "cd $dirOld; mv $baseOld $baseNew";
    }
    if (system($cmd)) {
        print STDERR "ERROR: Problem renaming $filename to $newName\n";
        exit 1;
    }
    return 0;
}

main();

######################################################################
# End of Windows self-start wrapper

__END__
:endofperl
